<?php

namespace Mbl\Statistics;

require_once __DIR__ . '/modules/visits.php';

add_action('admin_enqueue_scripts', 'Mbl\Statistics\enqueue_profile_styles');
function enqueue_profile_styles()
{
    $current_screen = get_current_screen();
    if ($current_screen->id === 'user-edit') {
        wp_enqueue_style('custom-admin-styles',
            MBS_PLUGIN_URI . '/assets/mbs-profile.css?v=' . (isset($_GET['debugger']) ? time() : MBS_VERSION)
        );
        wp_enqueue_script(
            'custom-admin-scripts',
            MBS_PLUGIN_URI . '/assets/mbs-profile.js?v=' . MBS_VERSION,
            ['jquery']
        );
        wp_localize_script(
            'custom-admin-scripts',
            'mbs_data',
            ['show' => isset($_GET['mbs_show']) ? 1 : 0]
        );
    }
}

add_action('mbl_material_passed', 'Mbl\Statistics\lesson_passed', 10, 2);
function lesson_passed($postId, $isPassed)
{
    global $wpdb;
    $table = "{$wpdb->prefix}memberlux_lesson_passed";
    $userId = get_current_user_id();
    $row = $wpdb->get_row(
        $wpdb->prepare("SELECT * FROM `$table` WHERE user_id = %d AND post_id = %d", [$userId, $postId])
    );
    if (!$isPassed) {
        if (!is_null($row)) {
            $wpdb->delete($table, ['user_id' => $userId, 'post_id' => $postId]);
        }
    } else  {
        $date = (new \DateTime('now', wp_timezone()))->format('Y-m-d H:i:s');
        if (!is_null($row)) {
            $wpdb->update($table, ['datetime' => $date], ['user_id' => $userId, 'post_id' => $postId]);
        } else {
            $wpdb->insert($table, ['user_id' => $userId, 'post_id' => $postId, 'datetime' => $date]);
        }
    }
}

function certificatesList($user): array
{
    try {
        $certificates = array_map(
            function ($certificate) {
                return [
                    'name' => $certificate->getCertificateName(),
                    'date' => $certificate->getDateIssue(),
                    'download_pdf' => certUrl($certificate->id, 'pdf', 'download'),
                ];
            },
            \Certificate::getCustomerCertificates($user->ID)
        );
    } catch (\Exception $e) {
        $certificates = [];
    }

    return $certificates;
}
