<?php

namespace Mbl\Statistics;

class Core
{
    public function __construct()
    {
    }

    public function run()
    {
        require_once MBS_PLUGIN_PATH . '/admin/functions.php';
        add_action('init', [$this, 'init']);
        add_action('plugins_loaded', [$this, 'loaded']);
        require_once MBS_PLUGIN_PATH . '/admin/rest.php';
    }

    function loaded()
    {
        if (!$this->_pluginEnabled()) {
            return;
        }
        add_action('load-user-edit.php', 'Mbl\Statistics\write_user_visit');
        if (!isset($_GET['add-to-cart'])) {
            add_action('wp', 'Mbl\Statistics\write_user_visit');
        } else {
            add_action('init', 'Mbl\Statistics\write_user_visit');
        }
    }

    public function init()
    {
        if ($this->_pluginEnabled()) {
            require_once MBS_PLUGIN_PATH . '/admin/main.php';
        } elseif (is_admin()) {
            add_action('admin_notices', array($this, 'notActiveNotice'));
        }
    }

    public function install()
    {
        defined('WP_MEMBERSHIP_VERSION') || exit;
        global $wpdb;
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        $table_name = $wpdb->get_blog_prefix() . 'memberlux_stats_visit';
        $sql = "CREATE TABLE `$table_name` (
            `id` bigint(20) PRIMARY KEY AUTO_INCREMENT,
            `user_id` bigint(20) NOT NULL,
            `datetime` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `url` text NOT NULL,
            `ip` varchar(20) NULL,
            `country_name` varchar(100) NULL,
            `country_code` varchar(20) NULL,
            `browser` varchar(100) NULL,
            `os` varchar(100) NULL,
            `device` varchar(255) NULL,
            `brandname` varchar(255) NULL,
            `model` varchar(255) NULL,
            `user_agent` text NULL,
            `title` varchar(255) NULL,
            `count` int unsigned DEFAULT 1 
        ) 
        DEFAULT CHARACTER SET utf8
        DEFAULT COLLATE utf8_general_ci;";
        $table_name_passed = $wpdb->get_blog_prefix() . 'memberlux_lesson_passed';
        $sql_passed = "CREATE TABLE `$table_name_passed` (
            `user_id` bigint(20) NOT NULL,
            `post_id` bigint(20) NOT NULL,
            `datetime` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            UNIQUE KEY `user_post_key` (`user_id`,`post_id`)
        ) 
        DEFAULT CHARACTER SET utf8
        DEFAULT COLLATE utf8_general_ci;";
        dbDelta($sql);
        dbDelta($sql_passed);
    }

    public function notActiveNotice(): void
    {
        $class = 'notice notice-error';
        $message = __('МОДУЛЬ СТАТИСТИКИ ПОЛЬЗОВАТЕЛЕЙ не активирован!', 'mbl_admin');
        if (!defined('WP_MEMBERSHIP_VERSION')) {
            $message .= ' ' . __('Не установлен плагин MEMBERLUX.', 'mbl_admin');
        } elseif (version_compare(WP_MEMBERSHIP_VERSION, '2.47', '<')) {
            $message .= ' ' . __('Устаревшая версия MEMBERLUX.', 'mbl_admin');
        } elseif (!$this->_pluginKeyActive()) {
            $message .= ' ' . __('Ключ не активирован.', 'mbl_admin');
        }
        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
    }

    private function _pluginEnabled(): bool
    {
        return defined('WP_MEMBERSHIP_VERSION')
            && !version_compare(get_option('wpm_version'), '2.47', '<')
            && $this->_pluginKeyActive();
    }

    private function _pluginKeyActive(): bool
    {
        $activeKeys = wpm_array_pluck(wpm_array_get(wpm_get_key_data(), 'keys.active', array()), 'type');

        if (count(wpm_array_filter($activeKeys, 'fullpackaccess'))) {
            return true;
        }

        return (bool)count(wpm_array_filter($activeKeys, 'statistics-users'));
    }
}
