<?php

namespace Mbl\Statistics;

use WP_User;

class Courses
{
    private WP_User $user;

    /**
     * @param WP_User | int $user
     */
    public function __construct($user)
    {
        if (is_int($user)) {
            $this->user = get_user_by('ID', $user);
        } else {
            $this->user = $user;
        }
    }

    public function list(): array
    {
        global $wpdb;
        $sql = "SELECT DISTINCT `cat`.`term_id` cat_id, `tt_lvl`.`term_id` wpm_level" . PHP_EOL .
            "FROM `$wpdb->terms` AS cat" . PHP_EOL .
            "INNER JOIN `$wpdb->term_taxonomy` AS tt" . PHP_EOL .
            "\tON tt.term_id = cat.term_id AND tt.taxonomy = 'wpm-category'" . PHP_EOL .
            "INNER JOIN `$wpdb->term_relationships` AS cat_r" . PHP_EOL .
            "\tON cat_r.term_taxonomy_id = tt.term_taxonomy_id" . PHP_EOL .
            "INNER JOIN `$wpdb->term_relationships` AS lvl_r" . PHP_EOL .
            "\tON lvl_r.object_id = cat_r.object_id" . PHP_EOL .
            "INNER JOIN `$wpdb->term_taxonomy` AS tt_lvl" . PHP_EOL .
            "\tON tt_lvl.term_taxonomy_id = lvl_r.term_taxonomy_id AND tt_lvl.taxonomy = 'wpm-levels'" . PHP_EOL .
            "ORDER BY `cat`.`name` ASC;";
        $result = $wpdb->get_results($sql, ARRAY_A);
        $grouped = [];
        foreach ($result as $item) {
            if (!isset($grouped[$item['cat_id']])) {
                $grouped[$item['cat_id']] = [];
            }
            $grouped[$item['cat_id']][] = $item['wpm_level'];
        }
        /**
         * Caching courses
         */
        $allLessons = new \WP_Query([
            'post_type' => 'wpm-page',
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'tax_query' => [
                [
                    'taxonomy' => 'wpm-category',
                    'operator' => 'EXISTS',
                ]
            ],
        ]);
        $allLessonsStore = [];
        foreach ($allLessons->posts as $post) {
            $_categories = wp_get_post_terms($post->ID, 'wpm-category');
            foreach ($_categories as $term) {
                $allLessonsStore[$term->term_id][] = $post;
            }
        }
        get_terms(['taxonomy' => 'wpm-category', 'hide_empty' => false]);
        $allCourses = get_terms(['taxonomy' => 'wpm-category', 'hide_empty' => false, 'childless' => true]);
        $levels = wpm_get_all_user_accesible_levels($this->user->ID);
        $categories = [];
        foreach ($allCourses as $term) {
            $term_id = $term->term_id;
            $lessons = $allLessonsStore[$term_id] ?? [];
            $visibleLessonsCount = 0;
            $totalAvailable = 0;
            $show_all_auto_training_lessons = wpm_get_design_option('page.show_all') == 'on';
            foreach ($lessons as $post) {
                $postAccessLevels = wp_get_post_terms($post->ID, 'wpm-levels');
                $isAccess = empty($postAccessLevels) || !empty(array_intersect(
                        array_map(fn($i) => $i->term_id, $postAccessLevels),
                        $levels
                    ));
                
                $hide_for_no_access = true;
                foreach ($postAccessLevels as $level) {
                    $termId = $level->term_id;
                    $termMeta = get_option("taxonomy_term_$termId");
                    if (!isset($termMeta['hide_for_no_access']) || $termMeta['hide_for_no_access'] != 'hide') {
                        $hide_for_no_access = false;
                        break;
                    }
                }

                if ($isAccess) {
                    $totalAvailable++;
                }

                if (!$hide_for_no_access || $isAccess || $show_all_auto_training_lessons) {
                    $visibleLessonsCount++;
                }
            }
            $lessonsNoAccess = $term->count - $totalAvailable;
            if ($lessonsNoAccess === 0) {
                $availability = 'available';
            } elseif ($totalAvailable > 0) {
                $availability = 'partially';
            } else {
                $availability = 'no_available';
            }
            if (isset($grouped[$term_id])) {
                $cat_levels = $grouped[$term_id];
                $courseLevels = implode(', ', array_map(fn($id) => get_term($id, 'wpm-levels')->name, $cat_levels));
            } else {
                $courseLevels = '';
            }
            $meta = get_term_meta($term_id, 'wmp_term_progress', true);
            $totalCount = $visibleLessonsCount;
            $progress = $totalCount ? min(100, ceil(count(wpm_array_get($meta, $this->user->ID, array())) / $totalCount * 100)) : 0;
            $hasParent = !!$term->parent;
            $child = $term;
            $name = $term->name;
            while ($hasParent) {
                $parent = get_term($child->parent, 'wpm-category');
                $name = "$parent->name → $name";
                $child = $parent;
                $hasParent = !!$child->parent;
            }
            $categories[] = [
                'name' => $name,
                'term' => $term,
                'link' => get_term_link($term),
                'availability' => $availability,
                'progress' => $progress,
                'levels' => $courseLevels,
            ];
        }

        return $categories;
    }
}
