<?php

namespace Mbl\Statistics;

use WP_Term;
use WP_User;

class Lessons
{
    private WP_User $user;
    private WP_Term $term;

    /**
     * @param WP_User | int $user
     * @param WP_Term | int $term
     */
    public function __construct($user, $term)
    {
        if (is_int($user)) {
            $this->user = get_user_by('ID', $user);
        } else {
            $this->user = $user;
        }
        if (is_int($term)) {
            $this->term = get_term($term, 'wpm-category');
        } else {
            $this->term = $term;
        }
    }

    public function list(): array
    {
        $user = $this->user;
        $lessons = [];
        $number = 1;
        add_filter('posts_join', [$this, 'join']);
        add_filter('posts_fields', [$this, 'fields']);
        $posts = new \WP_Query([
            'post_type' => 'wpm-page',
            'posts_per_page' => -1,
            'post_status' => 'publish',
            'tax_query' => array(
                array(
                    'taxonomy' => 'wpm-category',
                    'field' => 'term_id',
                    'terms' => $this->term->term_id
                )
            ),
            'orderby' => 'menu_order title',
            'order' => 'ASC'
        ]);
        remove_filter('posts_join', [$this, 'join']);
        remove_filter('posts_fields', [$this, 'fields']);
        $show_all_auto_training_lessons = wpm_get_design_option('page.show_all') == 'on';
        $is_auto_training = wpm_is_autotraining($this->term->term_id);
        $prevIsPassed = false;
        foreach ($posts->posts as $post) {
            $postMeta = get_post_meta($post->ID, '_wpm_page_meta', true);
            $hasHomework = wpm_array_get($postMeta, 'is_homework') == 'on';
            $passed = get_user_meta($user->ID, 'passed_materials', true);
            $passed = is_array($passed) ? $passed : [];
            if ($hasHomework) {
                $homework = new Homework($post, $user->ID);
                $responseStatus = $homework->getResponseStatus();
                if ($responseStatus === 'opened') {
                    $homeworkStatus = 'На проверке';
                } elseif ($responseStatus === 'rejected') {
                    $homeworkStatus = 'Отклонено';
                } elseif ($responseStatus === 'accepted' || $responseStatus === 'approved') {
                    $homeworkStatus = 'Выполнено';
                } else {
                    $homeworkStatus = $responseStatus ?? 'Не отправлено';
                }

                $isPassed = $homework->isCompleted();
                $date = $homework->date();
            } else {
                $homeworkStatus = '—';
                $isPassed = wpm_array_get($passed, $post->ID, false);
                $date = $post->datetime ?? '';
            }

            if (!empty($date)) {
                $date = date('d.m.Y H:i:s', strtotime($date));
            }

            $postAccessLevels = wp_get_post_terms($post->ID, 'wpm-levels');
            $isAccess = empty($postAccessLevels) || !empty(array_intersect(
                    array_map(fn($i) => $i->term_id, $postAccessLevels),
                    wpm_get_all_user_accesible_levels($user->ID)
                ));

            $hide_for_no_access = true;
            foreach ($postAccessLevels as $level) {
                $termId = $level->term_id;
                $termMeta = get_option("taxonomy_term_$termId");
                if (!isset($termMeta['hide_for_no_access']) || $termMeta['hide_for_no_access'] != 'hide') {
                    $hide_for_no_access = false;
                    break;
                }
            }

            $isVisible = !$hide_for_no_access || $isAccess;

            if ($is_auto_training) {
                $isVisible = $isVisible && ($show_all_auto_training_lessons || $number === 1 || $prevIsPassed);
            }

            $lessons[] = [
                'number' => $number,
                'name' => $post->post_title,
                'link' => esc_attr(get_post_permalink($post)),
                'is_passed' => $isPassed,
                'is_visible' => $isVisible,
                'is_access' => $isAccess,
                'date' => $date,
                'homeworkStatus' => $homeworkStatus,
                'accessLevels' => implode(', ', array_map(fn($i) => $i->name, $postAccessLevels)),
            ];
            $number++;
            $prevIsPassed = $isPassed;
        }

        return $lessons;
    }

    public function join($join): string
    {
        global $wpdb;
        return $join . " LEFT JOIN `{$wpdb->prefix}memberlux_lesson_passed` ON" .
            " `{$wpdb->prefix}memberlux_lesson_passed`.post_id = `$wpdb->posts`.ID"
            . " AND `{$wpdb->prefix}memberlux_lesson_passed`.user_id = {$this->user->ID}";
    }

    public function fields($fields): string
    {
        global $wpdb;
        return $fields . ", `{$wpdb->prefix}memberlux_lesson_passed`.datetime";
    }
}
