<?php

namespace Mbl\Statistics;

use DateTime;
use MBLStats;
use MBLStatsDetector;
use WP_User;

class Visit
{
    private string $table;

    public function __construct()
    {
        global $wpdb;
        $this->table = "{$wpdb->get_blog_prefix()}memberlux_stats_visit";
    }

    public function write()
    {
        if (!is_user_logged_in()) {
            return;
        }
        $currentUser = wp_get_current_user();
        /*
        if (!in_array('customer', $currentUser->roles)) {
            return;
        }
        */
        if (wp_doing_cron() || wp_doing_ajax() || wp_is_json_request() || defined('XMLRPC_REQUEST')) {
            return;
        }
        if (!isset($_SERVER['HTTP_ACCEPT']) || strpos($_SERVER['HTTP_ACCEPT'], 'text/html') === false) {
            return;
        }
        global $wpdb;
        $detector = MBLStatsDetector::detect();
        $visitorCountryInfo = MBLStats::getCountryInfo();
        $url = $this->url();
        if (!empty($_GET['add-to-cart'])) {
            $title = get_post($_GET['add-to-cart'])->post_title;
        } else if (is_admin() && $GLOBALS['pagenow'] === 'user-edit.php') {
            $title = 'Профиль';
        } else if (get_query_var('wpm-page') === 'activation') {
            $title = 'Активация';
        } else {
            add_filter('document_title_parts', [$this, 'title']);
            $title = wp_get_document_title();
            remove_filter('document_title_parts', [$this, 'title']);
        }
        $statistics = [
            'datetime' => (new DateTime('now', wp_timezone()))->format('Y-m-d H:i:s'),
            'ip' => MBLStats::getIP(),
            'country_name' => $visitorCountryInfo->geoplugin_countryName ?? null,
            'country_code' => $visitorCountryInfo->geoplugin_countryCode ?? null,
            'browser' => implode(
                ' ',
                array_filter([$detector->getClient('name'), $detector->getClient('version')])
            ),
            'os' => implode(
                ' ',
                array_filter([
                    $detector->getOs('name'),
                    $detector->getOs('version'),
                    $detector->getOs('platform')
                ])
            ),
            'device' => $detector->getDeviceName(),
            'brandname' => $detector->getBrandName(),
            'model' => $detector->getModel(),
            'user_agent' => $detector->getUserAgent(),
            'title' => $title
        ];
        $data = $wpdb->get_row($wpdb->prepare(
            "SELECT `id`, `count` FROM `$this->table` WHERE user_id = %d AND url = %s",
            [$currentUser->ID, $url]
        ), ARRAY_A);
        if (is_null($data)) {
            $statistics['user_id'] = $currentUser->ID;
            $statistics['url'] = $url;
            $statistics['count'] = 1;
            $wpdb->insert($this->table, $statistics);
        } else {
            $statistics['count'] = $data['count'] + 1;
            $wpdb->update($this->table, $statistics, ['id' => $data['id']]);
        }
    }

    public function url(): string
    {
        $schema = is_ssl() ? 'https' : 'http';
//        $parsedUrl = parse_url("$schema://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}");
//        $url = $parsedUrl['scheme'] . '://' . $parsedUrl['host'] . $parsedUrl['path'];
        $url = "$schema://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $parsedUrl = parse_url($url);

        return (substr($url, -1) !== '/' && empty($parsedUrl['query'])) ? $url . '/' : $url;
    }

    /**
     * @param WP_User | int $user
     *
     * @return array
     */
    public function userStat($user): array
    {
        if ($user instanceof WP_User) {
            $user_id = $user->ID;
        } else {
            $user_id = $user;
        }
        global $wpdb;
        $sql = $wpdb->prepare("SELECT * FROM `$this->table` WHERE user_id = %d ORDER BY datetime DESC", [$user_id]);

        return $wpdb->get_results($sql, ARRAY_A);
    }

    public function title($title)
    {
        $title['site'] = '';
        return $title;
    }
}
